require "Window"
require "CraftingLib"
require "HousingLib"

local FLexFurnish = {}

local nDecorPerPage = 20
local tTestPhrases = {
	["PH_"] = true,
	["PH Ext"] = true,
	["Placeholder"] = true,
	["PRP_"] = true,
	["[TEST]"] = true,
}
local tQualityColors = {
	[Item.CodeEnumItemQuality.Inferior] 		= "ItemQuality_Inferior",
	[Item.CodeEnumItemQuality.Average] 			= "ItemQuality_Average",
	[Item.CodeEnumItemQuality.Good] 			= "ItemQuality_Good",
	[Item.CodeEnumItemQuality.Excellent] 		= "ItemQuality_Excellent",
	[Item.CodeEnumItemQuality.Superb] 			= "ItemQuality_Superb",
	[Item.CodeEnumItemQuality.Legendary] 		= "ItemQuality_Legendary",
	[Item.CodeEnumItemQuality.Artifact]		 	= "ItemQuality_Artifact",
}
local tFactionEnemy = {
	[170] = 167, -- Dominion Vendor to Exile Player
	[171] = 166, --Exile Vendor to Dominion Player
}

function FLexFurnish:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
	
	self.nColorId = 0
	self.tTypes = {}
	self.tDecor = {}
	self.tVendors = {}
	
	self.tTypeIds = {}
	self.tFilteredIds = {}
	
    return o
end

function FLexFurnish:Init()
    Apollo.RegisterAddon(self, false, "", {})
end

function FLexFurnish:OnLoad()
	Apollo.LoadSprites("FLexFurnishSprites.xml", "FLexFurnishSprites")
	self.xmlDoc = XmlDoc.CreateFromFile("FLexFurnish.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function FLexFurnish:OnDocLoaded()
	if self.xmlDoc == nil then
		return
	end
	
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	Apollo.RegisterEventHandler("ToggleFLexFurnish", "OnToggleFLexFurnish", self)
	
	Apollo.RegisterEventHandler("InvokeVendorWindow", "OnInvokeVendorWindow", self)
	Apollo.RegisterEventHandler("VendorItemsUpdated", "OnVendorItemsUpdated", self)
	
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "FLexFurnishForm", nil, self)
	self.wndMain:Show(false, true)
	
	self.wndTypeButton = self.wndMain:FindChild("Filters:Type")
	self.wndTypeDropdown = self.wndTypeButton:FindChild("Filters:Type:Dropdown")
	self.wndTypeList = self.wndTypeDropdown:FindChild("List")
	
	self.wndTextFilter = self.wndMain:FindChild("Filters:Text:EditBox")
	
	self.wndDecorList = self.wndMain:FindChild("Decor:List")
	self.wndDecorColors = self.wndMain:FindChild("Decor:Colors")
	self.wndDecorPage = self.wndMain:FindChild("Decor:Page")
	self.wndPageCurr = self.wndDecorPage:FindChild("Current")
	self.wndPageMax = self.wndDecorPage:FindChild("Max")
	
	self.wndInfo = self.wndMain:FindChild("Info")
	self.wndInfoItem = self.wndInfo:FindChild("Item")
	self.wndInfoIcon = self.wndInfoItem:FindChild("Icon")
	self.wndInfoItemName = self.wndInfoItem:FindChild("ItemName")
	self.wndInfoDecorName = self.wndInfo:FindChild("DecorName")
	self.wndInfoObtain = self.wndInfo:FindChild("Obtain")
	self.wndInfoModel = self.wndInfo:FindChild("Model")
	
	self:DrawDecorList()
	self:DrawDecorColors()
	self:DrawTypeList()
	
	if self.tTypes[1] then
		self.tTypeIds = self.tTypes[1].tDecorIds
	end
	
	self:OnSetPage(1)
end

--Events
function FLexFurnish:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "fLex Furnish", {"ToggleFLexFurnish", "", "FLexFurnishSprites:LampIcon"})
end

function FLexFurnish:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementRegister", {strName = "fLex Furnish", nSaveVersion = 3})
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "fLex Furnish"})
end

function FLexFurnish:OnToggleFLexFurnish()
	if not self.wndMain:IsVisible() then
		self.wndMain:Show(true, false)
	else
		self.wndMain:Close()
	end
end

function FLexFurnish:OnInvokeVendorWindow(unit)
	self.unitVendor = unit
end

function FLexFurnish:OnVendorItemsUpdated()
	local strVendorName = self.unitVendor:GetName()
	local tVendor = self.tVendors[strVendorName]
	local strVersion = self:GetVersionString()
	
	if tVendor then
		local strVendorVersion = tVendor.strVersion
		
		if strVendorVersion == strVersion then
			return
		end
	end
	
	local tItems = self.unitVendor:GetVendorItems()
	local bHadDecor = false
	local nCount = #tItems
	
	if not self.tVendors[strVendorName] then
		self.tVendors[strVendorName] = {}
	end
	
	for i = 1, nCount do
		local tItem = tItems[i]
		local item = tItem.itemData
		local nItemType = item:GetItemType()
		
		if nItemType == 155 then
			local nDecorId = item:GetHousingDecorInfoId()
			
			if not self.tDecor[nDecorId] then
				self.tDecor[nDecorId] = {}
			end
			
			if not self.tDecor[nDecorId].tVendors then
				self.tDecor[nDecorId].tVendors = {}
			end
			
			if not self.tDecor[nDecorId].tVendors[strVendorName] then
				self.tDecor[nDecorId].tVendors[strVendorName] = {}
			end
						
			self.tDecor[nDecorId].strItemName = tItem.strName
			self.tDecor[nDecorId].nItemId = tItem.idItem
			self.tDecor[nDecorId].tVendors[strVendorName].nCost = tItem.tPriceInfo.nAmount1
			
			if tItem.tPriceInfo.eCurrencyType1 then
				self.tDecor[nDecorId].tVendors[strVendorName].nCurrency = tItem.tPriceInfo.eCurrencyType1
			elseif tItem.tPriceInfo.itemExchange1 then
				self.tDecor[nDecorId].tVendors[strVendorName].nExchangeItemId = tItem.tPriceInfo.itemExchange1:GetItemId()
			end
			
			if tItem.idPrereq then
				self.tDecor[nDecorId].tVendors[strVendorName].nPrereqId = tItem.idPrereq
			end
			
			bHadDecor = true
		end
	end
	
	if bHadDecor then
		self.tVendors[strVendorName].nFaction = self.unitVendor:GetFaction()
		self.tVendors[strVendorName].tLocation = {
			["nZone"] = GameLib.GetCurrentZoneMap().id,
			["tPos"] = self.unitVendor:GetPosition()
		}
	end
	
	self.tVendors[strVendorName].strVersion = self:GetVersionString()
end

--Save
function FLexFurnish:OnSave(eType)
	if eType == GameLib.CodeEnumAddonSaveLevel.General then
		local tSavedData = {}
		
		if self.tDecor then
			tSavedData.tDecor = self.tDecor
		end
		
		if self.tVendors then
			tSavedData.tVendors = self.tVendors
		end
		
		if self.tTypes then
			tSavedData.tTypes = self.tTypes
		end
		
		if self.strVersion then
			tSavedData.strVersion = self.strVersion
		end
		
		return tSavedData
	end
end

function FLexFurnish:OnRestore(eType, tSavedData)
	if eType == GameLib.CodeEnumAddonSaveLevel.General then
		if tSavedData.tDecor then
			self.tDecor = tSavedData.tDecor
		end
		
		if tSavedData.tVendors then
			self.tVendors = tSavedData.tVendors
		end
		
		if tSavedData.tTypes then
			self.tTypes = tSavedData.tTypes
		end
		
		if tSavedData.strVersion then
			self.strVersion = tSavedData.strVersion
		end
	end
end

function FLexFurnish:UpdateHousingLists()
	if not HousingLib.IsOnMyResidence() then
		return
	end
	
	if self.strVersion ~= self:GetVersionString() then
		local tTypeList = HousingLib:GetDecorTypeList()
		local nCount = #tTypeList
		local tAllIds = {}
		self.tTypes = {}
		
		for i = 1, nCount do
			local tType = tTypeList[i]
			local bIsOld = (string.sub(tType.strLuaVariable, 1, 3) == "OLD")
			
			if not bIsOld and tType.nId and tType.strName ~= "" then
				self.tTypes[tType.nId] = {
					["nTypeId"] = tType.nId,
					["strTypeName"] = tType.strName,
					["tDecorIds"] = {}
				}
			end
		end
		
		local nDecorId = 1
		local nNil = 0
		
		while nNil < 1000 do
			local decor = HousingLib.PreviewVendorDecor(nDecorId)
			
			if decor ~= nil and decor:GetName() ~= "" then
				local strDecorName = decor:GetName()
				local nTypeId = decor:GetDecorType()
				local bIsTest
				
				for strTest, _ in pairs(tTestPhrases) do
					local strSub = string.sub(strDecorName, 1, #strTest)
					
					if strSub == strTest then
						bIsTest = true
					end
				end
				
				if self.tTypes[nTypeId] and not bIsTest then
					if not self.tDecor[nDecorId] then
						self.tDecor[nDecorId] = {}
					end
					
					self.tDecor[nDecorId].strDecorName = strDecorName
					self.tDecor[nDecorId].nTypeId = nTypeId
					table.insert(self.tTypes[nTypeId].tDecorIds, nDecorId)
					table.insert(tAllIds, nDecorId)
					
					nNil = 0
				end
			else
				nNil = nNil + 1
			end
			
			nDecorId = nDecorId + 1
		end
		
		table.sort(self.tTypes, function(a, b) if a and b then return a.strTypeName < b.strTypeName elseif a then return true end return false end)
		table.insert(self.tTypes, 1, { ["nTypeId"] = 0, ["strTypeName"] = Apollo.GetString("HousingDecorate_AllTypes"), ["tDecorIds"] = tAllIds } )
		
		nCount = #self.tTypes
		
		for i = 1, nCount do
			table.sort(self.tTypes[i].tDecorIds, function(a, b) return self.tDecor[a].strDecorName < self.tDecor[b].strDecorName end)
		end
		
		self.strVersion = self:GetVersionString()
	end
	
	local strVendorName = Apollo.GetString("HousingDecorate_VendorLabel")
	local tVendor = self.tVendors[strVendorName]
	
	if not self.tVendors[strVendorName] then
		self.tVendors[strVendorName] = {}
	end
	
	local tDecorList = HousingLib.GetDecorCatalogList()
	nCount = #tDecorList
	
	for i = 1, nCount do
		local tDecor = tDecorList[i]
		local nDecorId = tDecor.nId
		
		if not self.tDecor[nDecorId] then
			self.tDecor[nDecorId] = {}
		end
		
		if not self.tDecor[nDecorId].tVendors then
			self.tDecor[nDecorId].tVendors = {}
		end
		
		if not self.tDecor[nDecorId].tVendors[strVendorName] then
			self.tDecor[nDecorId].tVendors[strVendorName] = {}
		end
		
		self.tDecor[nDecorId].strDecorName = tDecor.strName
		self.tDecor[nDecorId].tVendors[strVendorName].nCurrency = tDecor.eCurrencyType
		self.tDecor[nDecorId].tVendors[strVendorName].nCost = tDecor.nCost
	end
	
	self.tVendors[strVendorName].tLocation = {
		["nZone"] = GameLib.GetCurrentZoneMap().id,
		["tPos"] = {
			["x"] = 1573.5100097656,
			["y"] = -707.93359375,
			["z"] = 1322.6600341797,
		}
	}
	
	self:DrawTypeList()
	
	if self.tTypes[1] then
		self.tTypeIds = self.tTypes[1].tDecorIds
	end
	
	self:OnSetPage(1)
end

function FLexFurnish:UpdateArchitectList()
	local tKnown = CraftingLib.GetKnownTradeskills()
	local nCount = #tKnown
	local bIsArchitect = false
	
	for i = 1, nCount do
		if tKnown[i].eId == 17 then
			bIsArchitect = true
			break
		end
	end
	
	if not bIsArchitect then
		return
	end

	local tSchematics = CraftingLib.GetSchematicList(17, nil, CraftingLib.CodeEnumTradeskillTier.Zero, true)
	nCount = #tSchematics
	
	for i = 1, nCount do
		local tSchematic = tSchematics[i]
		
		if tSchematic.eItemType == 155 then
			local tInfo = CraftingLib.GetSchematicInfo(tSchematic.nSchematicId)
			local item = tInfo.itemOutput
			local nDecorId = item:GetHousingDecorInfoId()
			
			if not self.tDecor[nDecorId] then
				self.tDecor[nDecorId] = {}
			end
			
			self.tDecor[nDecorId].nItemId = item:GetItemId()
			self.tDecor[nDecorId].strItemName = item:GetName()
			
			local tMats = tInfo.arMaterials
			local tMaterials = {}
			local nCount2 = #tMats
			
			for i = 1, nCount2 do
				local tMaterial = tMats[i]
				local nMaterialId = tMaterial.itemMaterial:GetItemId()
				
				tMaterials[nMaterialId] = tMaterial.nNeeded
			end
			
			self.tDecor[nDecorId].tMaterials = tMaterials
			
			local tSubs = tInfo.tSubRecipes
			nCount2 = #tSubs
			
			for i = 1, nCount2 do
				local item2 = tSubs[i].itemOutput
				local nDecorId2 = item2:GetHousingDecorInfoId()
				
				if not self.tDecor[nDecorId2] then
					self.tDecor[nDecorId2] = {}
				end
				
				self.tDecor[nDecorId2].nItemId = item2:GetItemId()
				self.tDecor[nDecorId2].strItemName = item2:GetName()
				self.tDecor[nDecorId2].tMaterials = tMaterials
			end
		end
	end
end

function FLexFurnish:DrawTypeList()
	local nTypes = #self.tTypes
	self.wndTypeList:DestroyChildren()
	
	for i = 1, nTypes do
		local tType = self.tTypes[i]
		local nDecor = #tType.tDecorIds
		
		if nDecor > 0 then
			self:DrawType(tType)
		end
	end

	self.wndTypeList:ArrangeChildrenVert(Window.CodeEnumArrangeOrigin.LeftOrTop)
end

function FLexFurnish:DrawType(tType)
	local wndType = Apollo.LoadForm(self.xmlDoc, "Type", self.wndTypeList, self)
	
	wndType:SetText(String_GetWeaselString(Apollo.GetString("HealthBar_HealthTextFullShield"), tType.strTypeName, #tType.tDecorIds))
	wndType:SetData(tType.tDecorIds)
end

function FLexFurnish:DrawDecorList()
	self.wndDecorList:DestroyChildren()
	
	for i = 1, nDecorPerPage do
		local wndDecor = Apollo.LoadForm(self.xmlDoc, "Decor", self.wndDecorList, self)
		
		wndDecor:SetName("Decor" .. i)
	end
	
	self.wndDecorList:ArrangeChildrenTiles(Window.CodeEnumArrangeOrigin.LeftOrTop)
end

function FLexFurnish:DrawDecorColors()
	self.wndDecorColors:DestroyChildren()
	
	local tColors = HousingLib.GetDecorColorOptions()
	table.insert(tColors, 1, { id = 0, strName = Apollo.GetString("Housing_ColorShiftRemoveEntry"), strPreviewSwatch = "" })
	
	for idx, tColorShift in pairs(tColors) do
		local wndColor = Apollo.LoadForm(self.xmlDoc, "Color", self.wndDecorColors, self)
		local wndButton = wndColor:FindChild("Button")
		local wndSwatch = wndColor:FindChild("Swatch")
		
		wndSwatch:SetSprite(tColorShift.strPreviewSwatch)
		wndButton:SetTooltip(tColorShift.strName)
		wndButton:SetData(tColorShift.id)
	end
	
	self.wndDecorColors:ArrangeChildrenHorz(Window.CodeEnumArrangeOrigin.LeftOrTop)
end

function FLexFurnish:SetDecorColor(nColorId)
	if nColorId ~= nil then
		self.nColorId = nColorId
	end
	
	for i = 1, nDecorPerPage do
		local wndDecorModel = self.wndDecorList:FindChild("Decor" .. i .. ":Model")
		
		if wndDecorModel ~= nil and wndDecorModel:IsVisible() then
			wndDecorModel:SetDecorColor(self.nColorId)
		end
	end
end

function FLexFurnish:UpdateTextFilter()
	self.tFilteredIds = {}
	
	if not self.strTextFilter or #self.strTextFilter < 1 then
		self.wndTextFilter:SetText("Search")
		self:OnSetPage(1)
		
		return
	end
	
	local nCount = #self.tTypeIds
	
	for i = 1, nCount do
		local tDecor = self.tDecor[self.tTypeIds[i]]
		
		if string.match(string.lower(tDecor.strDecorName), string.lower(self.strTextFilter)) then
			table.insert(self.tFilteredIds, self.tTypeIds[i])
		end
	end
	
	self:OnSetPage(1)
end

function FLexFurnish:OnSetPage(nPage)
	local tDecorIds
	
	if #self.tFilteredIds > 0 then
		tDecorIds = self.tFilteredIds
	else
		tDecorIds = self.tTypeIds
	end
	
	for i = 1, nDecorPerPage do			
		local nDecorId = tDecorIds[(i + ((nPage - 1) * nDecorPerPage))]
		local tDecor = self.tDecor[nDecorId]
		local wndDecor = self.wndDecorList:FindChild("Decor" .. i)
		local wndDecorModel = wndDecor:FindChild("Model")
		local bIsVisible = wndDecor:IsVisible()
		
		if tDecor then
			wndDecor:SetData(nDecorId)
			wndDecor:FindChild("DecorName"):SetText(tDecor.strDecorName)
			wndDecorModel:SetDecorInfo(nDecorId)
			wndDecorModel:SetDecorColor(self.nColorId)
			
			if not bIsVisible then
				wndDecor:Show(true, true)
			end
		elseif bIsVisible then
			wndDecor:Show(false, true)
		end
	end
	
	self.nPage = nPage
	self.nTotalPages = math.ceil(#tDecorIds / nDecorPerPage)
	self.wndPageCurr:SetText(self.nPage)
	self.wndPageMax:SetText("/  " .. self.nTotalPages)
end

function FLexFurnish:GetVersionString()
	local tVersion = GameLib.GetVersionInfo()
	
	return string.format("%d.%d.%d", tVersion.nVersion, tVersion.nVersionMajor, tVersion.nVersionMinor)
end

function FLexFurnish:OnDecorButtonCheck(wndHandler, wndControl, eMouseButton)
	local nDecorId = wndControl:GetData()
	self.wndInfoObtain:DestroyChildren()
	
	if nDecorId then
		local tDecor = self.tDecor[nDecorId]
		local bIsItemVisible = self.wndInfoItem:IsVisible()
		local bIsObtainVisible = self.wndInfoObtain:IsVisible()
		
		self.wndInfoDecorName:SetText(tDecor.strDecorName)
		
		if tDecor.nItemId then
			local item = Item.GetDataFromId(tDecor.nItemId)
			self.wndInfoIcon:GetWindowSubclass():SetItem(item)
			self.wndInfoItemName:SetText(tDecor.strItemName)
			self.wndInfoItemName:SetTextColor(tQualityColors[item:GetItemQuality()])
			
			if not bIsItemVisible then
				self.wndInfoItem:Show(true, true)
			end
		elseif bIsItemVisible then
			self.wndInfoItem:Show(false, true)
		end
		
		if tDecor.tVendors or tDecor.tMaterials then
			if tDecor.tVendors then
				local unitPlayer = GameLib.GetPlayerUnit()
				
				for strVendorName, tRequirements in pairs(tDecor.tVendors) do
					local nFaction = self.tVendors[strVendorName].nFaction
					local wndVendor = Apollo.LoadForm(self.xmlDoc, "Vendor", self.wndInfoObtain, self)
					local wndCost = wndVendor:FindChild("Cost")
					local wndName = wndVendor:FindChild("Name")
					
					wndName:SetText(strVendorName)
					wndName:SetTextColor((tFactionEnemy[nFaction] == unitPlayer:GetFaction()) and "Reddish" or "UI_TextHoloBodyCyan")
					
					if tRequirements.nCurrency then
						wndCost:SetMoneySystem(tRequirements.nCurrency)
						wndCost:SetTextColor((GameLib.GetPlayerCurrency(tRequirements.nCurrency):GetAmount() >= tRequirements.nCost) and "UI_TextHoloBodyCyan" or "Reddish")
					elseif tRequirements.nExchangeItemId then
						local item = Item.GetDataFromId(tRequirements.nExchangeItemId)
						
						wndCost:SetExchangeItem(item)
						wndCost:SetTextColor((item:GetBackpackCount() >= tRequirements.nCost) and "UI_TextHoloBodyCyan" or "Reddish")
					end
					
					wndCost:SetAmount(tRequirements.nCost)
					
					if tRequirements.nPrereqId then
						local wndRep = wndVendor:FindChild("Reputation")
						local tPrereqInfo = unitPlayer:GetPrereqInfo(tRequirements.nPrereqId) or nil
						
						if tPrereqInfo and tPrereqInfo.strText then
							wndRep:SetTextColor(tPrereqInfo.bIsMet and "UI_TextHoloBodyCyan" or "Reddish")
							wndRep:SetText(tPrereqInfo.strText)
						end
					end
				end
			end

			
			if tDecor.tMaterials then
				local wndArchitect = Apollo.LoadForm(self.xmlDoc, "Architect", self.wndInfoObtain, self)
				local wndMaterials = wndArchitect:FindChild("Materials")
				
				for nItemId, nCount in pairs(tDecor.tMaterials) do
					local wndMaterial = Apollo.LoadForm(self.xmlDoc, "Material", wndMaterials, self)
					local item = Item.GetDataFromId(nItemId)
					
					wndMaterial:GetWindowSubclass():SetItem(item)
					wndMaterial:FindChild("Count"):SetText(nCount)
					wndMaterial:SetData(item)
				end
				
				wndMaterials:ArrangeChildrenHorz(Window.CodeEnumArrangeOrigin.LeftOrTop)
			end
			
			if not bIsObtainVisible then
				self.wndInfoObtain:Show(true, true)
			end
		elseif bIsObtainVisible then
			self.wndInfoObtain:Show(false, true)
		end
		
		self.wndInfoObtain:ArrangeChildrenVert(Window.CodeEnumArrangeOrigin.LeftOrTop)
		self.wndInfoModel:SetDecorInfo(nDecorId)
		self.wndInfoModel:SetDecorColor(self.nColorId)
	end
end

function FLexFurnish:OnTypeToggleCheck()
	self.wndTypeDropdown:Show(true, true)
end

function FLexFurnish:OnTypeToggleUncheck()
	self.wndTypeDropdown:Show(false, true)
end

function FLexFurnish:OnTypeFilterCheck(wndHandler)
	self.tTypeIds = wndHandler:GetData()
	self:UpdateTextFilter()
	self.wndTypeButton:SetText(wndHandler:GetText())
	self.wndTypeButton:SetCheck(false)
	self.wndTypeDropdown:Show(false, true)
end

function FLexFurnish:OnVendorButtonClick(wndHandler, wndControl, eMouseButton)
	local strVendorName = wndControl:FindChild("Name"):GetText()
	local tLocation = self.tVendors[strVendorName].tLocation
	
	GameLib.SetNavPoint(tLocation.tPos, tLocation.nZone)
end

function FLexFurnish:OnCloseButtonClick(wndHandler, wndControl, eMouseButton)
	self:OnToggleFLexFurnish()
end

function FLexFurnish:OnFirstButtonClick()
	self:OnSetPage(1)
end

function FLexFurnish:OnPreviousButtonClick()
	if self.nPage > 1 then
		self:OnSetPage(self.nPage - 1)
	end
end

function FLexFurnish:OnNextButtonClick()
	if self.nPage < self.nTotalPages then
		self:OnSetPage(self.nPage + 1)
	end
end

function FLexFurnish:OnLastButtonClick()
	self:OnSetPage(self.nTotalPages)
end

function FLexFurnish:OnItemGenerateTooltip(wndHandler)
	Tooltip.GetItemTooltipForm(self, wndHandler, wndHandler:GetData(), {bSimple = true, strAppend = strAppend})
end

function FLexFurnish:OnUpdateHousingButtonClick()
	self:UpdateHousingLists()
end

function FLexFurnish:OnUpdateArchitectButtonClick()
	self:UpdateArchitectList()
end

function FLexFurnish:OnCurrentPageReturn(wndHandler, wndControl, strText)
	local nNewPage = tonumber(strText)
	
	if not nNewPage or nNewPage > self.nTotalPages or nNewPage < 1 then
		wndHandler:SetText(self.nPage)
		return
	end
	
	self:OnSetPage(nNewPage)
end

function FLexFurnish:OnColorButtonCheck(wndHandler)
	local nColorId = wndHandler:GetData()
	
	if nColorId ~= nil then
		self:SetDecorColor(nColorId)
	end
end

function FLexFurnish:OnTextFilterReturn(wndHandler, wndControl, strText)
	self.strTextFilter = strText
	self:UpdateTextFilter()
end

function FLexFurnish:OnTextFilterClear()
	self.wndTextFilter:SetText("Search")
	self.tFilteredIds = {}
	self.strTextFilter = ""
	self:OnSetPage(1)
end

local FLexFurnishInst = FLexFurnish:new()
FLexFurnishInst:Init()